import lalsimulation
from bilby.gw.source import _base_lal_cbc_fd_waveform
from lal import CreateDict


def lal_binary_black_hole_TIGER_PhenomP(
    frequency_array, mass_1, mass_2, luminosity_distance,
    a_1, tilt_1, phi_12, a_2, tilt_2, phi_jl,
    theta_jn, phase,
    dchi_0, dchi_1, dchi_2, dchi_3, dchi_4, dchi_5l, dchi_6, dchi_6l, dchi_7,
    dbeta_2, dbeta_3,
    dalpha_2, dalpha_3, dalpha_4, dalpha_5,
    **kwargs
):
    """
    A binary black hole waveform using lalsimulation with
    parameterized corrections in the TIGER framework. This
    interface supports the deviation in PN coefficients and
    legacy PhenomD/P coefficients in the intermediate-merger-ringdown
    phase: dbeta_2, dbeta_3, dalpha_2, dalpha_3, dalpha_4, dalpha_5.
    PhenomD/P does not support the deviation in the negative PN coefficient.
    Recommended waveform models: IMRPhenomD, IMRPhenomP,
    IMRPhenomPv2, IMRPhenomPv3, IMRPhenomHM, IMRPhenomPv3HM
    ----------
    Parameters
    ----------
    frequency_array: array_like
        The frequencies at which we want to calculate the strain
    mass_1: float
        The mass of the heavier object in solar masses
    mass_2: float
        The mass of the lighter object in solar masses
    luminosity_distance: float
        The luminosity distance in megaparsec
    a_1: float
        Dimensionless primary spin magnitude
    tilt_1: float
        Primary tilt angle
    phi_12: float
        Azimuthal angle between the component spins
    a_2: float
        Dimensionless secondary spin magnitude
    tilt_2: float
        Secondary tilt angle
    phi_jl: float
        Azimuthal angle between the total and orbital angular momenta
    theta_jn: float
        Orbital inclination
    phase: float
        The phase at coalescence
    dchi_0: float
        The non-GR inspiral parameter at 0PN
    dchi_1: float
        A non-GR inspiral parameter at 0.5PN
    dchi_2: float
        A non-GR inspiral parameter at 1PN
    dchi_3: float
        A non-GR inspiral parameter at 1.5PN
    dchi_4: float
        A non-GR inspiral parameter at 2PN
    dchi_5l: float
        A non-GR inspiral parameter at 2.5PN, logarithmic correction
    dchi_6: float
        A non-GR inspiral parameter at 3PN
    dchi_6l: float
        A non-GR inspiral parameter at 3PN, logarithmic correction
    dchi_7: float
        A non-GR inspiral parameter at 3.5PN
    dbeta_2: float
        A non-GR intermediate parameter in PhenomD/P
    dbeta_3: float
        A non-GR intermediate parameter in PhenomD/P
    dalpha_2: float
        A non-GR merger-ringdown parameter in PhenomD/P
    dalpha_3: float
        A non-GR merger-ringdown parameter in PhenomD/P
    dalpha_4: float
        A non-GR merger-ringdown parameter in PhenomD/P
    dalpha_5: float
        A non-GR merger-ringdown parameter in PhenomD/P
    kwargs: dict
        Optional keyword arguments
        Supported arguments:
            waveform_approximant
            reference_frequency
            minimum_frequency
            maximum_frequency
            catch_waveform_errors
            pn_spin_order
            pn_tidal_order
            pn_phase_order
            pn_amplitude_order
            mode_array:
                Activate a specific mode array and evaluate the model using those
                modes only.  e.g. waveform_arguments =
                dict(waveform_approximant='IMRPhenomHM', mode_array=[[2,2],[2,-2])
                returns the 22 and 2-2 modes only of IMRPhenomHM.  You can only
                specify modes that are included in that particular model.  e.g.
                waveform_arguments = dict(waveform_approximant='IMRPhenomHM',
                mode_array=[[2,2],[2,-2],[5,5],[5,-5]]) is not allowed because the
                55 modes are not included in this model.  Be aware that some models
                only take positive modes and return the positive and the negative
                mode together, while others need to call both.  e.g.
                waveform_arguments = dict(waveform_approximant='IMRPhenomHM',
                mode_array=[[2,2],[4,-4]]) returns the 22 and 2-2 of IMRPhenomHM.
                However, waveform_arguments =
                dict(waveform_approximant='IMRPhenomXHM', mode_array=[[2,2],[4,-4]])
                returns the 22 and 4-4 of IMRPhenomXHM.
    Returns
    -------
    dict: A dictionary with the plus and cross polarisation strain modes
    """
    waveform_kwargs = dict(
        waveform_approximant='IMRPhenomPv2', reference_frequency=50.0,
        minimum_frequency=20.0, maximum_frequency=frequency_array[-1],
        catch_waveform_errors=False, pn_spin_order=-1, pn_tidal_order=-1,
        pn_phase_order=-1, pn_amplitude_order=0)
    waveform_kwargs.update(kwargs)

    # If no waveform dictionary is passed, create one
    wf_dict = waveform_kwargs.get("lal_waveform_dictionary", CreateDict())

    # Relative shifts for inspiral phase PN coefficients (absolute value for dchi_minus2 and dchi_1)
    if dchi_0 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi0(wf_dict, float(dchi_0))
    if dchi_1 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi1(wf_dict, float(dchi_1))
    if dchi_2 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi2(wf_dict, float(dchi_2))
    if dchi_3 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi3(wf_dict, float(dchi_3))
    if dchi_4 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi4(wf_dict, float(dchi_4))
    if dchi_5l != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi5L(wf_dict, float(dchi_5l))
    if dchi_6 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi6(wf_dict, float(dchi_6))
    if dchi_6l != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi6L(wf_dict, float(dchi_6l))
    if dchi_7 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi7(wf_dict, float(dchi_7))

    # Relative shifts for intermediate phase coefficients (PhenomD/Pv2)
    if dbeta_2 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDBeta2(wf_dict, float(dbeta_2))
    if dbeta_3 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDBeta3(wf_dict, float(dbeta_3))

    # Relative shifts for merger-ringdown phase coefficients (PhenomD/Pv2)
    if dalpha_2 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDAlpha2(wf_dict, float(dalpha_2))
    if dalpha_3 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDAlpha3(wf_dict, float(dalpha_3))
    if dalpha_4 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDAlpha4(wf_dict, float(dalpha_4))
    if dalpha_5 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDAlpha5(wf_dict, float(dalpha_5))

    # Update waveform dictionary with TIGER coefficients
    waveform_kwargs["lal_waveform_dictionary"] = wf_dict

    return _base_lal_cbc_fd_waveform(
        frequency_array=frequency_array, mass_1=mass_1, mass_2=mass_2,
        luminosity_distance=luminosity_distance, theta_jn=theta_jn, phase=phase,
        a_1=a_1, a_2=a_2, tilt_1=tilt_1, tilt_2=tilt_2, phi_12=phi_12,
        phi_jl=phi_jl, **waveform_kwargs)


def lal_binary_black_hole_TIGER_PhenomX(
    frequency_array, mass_1, mass_2, luminosity_distance,
    a_1, tilt_1, phi_12, a_2, tilt_2, phi_jl,
    theta_jn, phase,
    dchi_minus2, dchi_minus1, dchi_0, dchi_1, dchi_2, dchi_3, dchi_4, dchi_5l, dchi_6, dchi_6l, dchi_7,
    db_1, db_2, db_3, db_4,
    dc_1, dc_2, dc_4, dc_l,
    **kwargs
):
    """
    A binary black hole waveform using lalsimulation with
    parameterized corrections in the TIGER framework. This
    interface supports the deviation in PN coefficients and
    the PhenomX post-inspiral coefficients: db_1, db_2, db_3,
    db_4, dc_1, dc_2, dc_4, dc_l.
    PhenomX supports the -1PN coefficient and -0.5PN coefficient.
    Recommended waveform models: IMRPhenomXAS, IMRPhenomXP,
    IMRPhenomXHM, IMRPhenomXPHM.
    ----------
    Parameters
    ----------
    frequency_array: array_like
        The frequencies at which we want to calculate the strain
    mass_1: float
        The mass of the heavier object in solar masses
    mass_2: float
        The mass of the lighter object in solar masses
    luminosity_distance: float
        The luminosity distance in megaparsec
    a_1: float
        Dimensionless primary spin magnitude
    tilt_1: float
        Primary tilt angle
    phi_12: float
        Azimuthal angle between the component spins
    a_2: float
        Dimensionless secondary spin magnitude
    tilt_2: float
        Secondary tilt angle
    phi_jl: float
        Azimuthal angle between the total and orbital angular momenta
    theta_jn: float
        Orbital inclination
    phase: float
        The phase at coalescence
    dchi_minus2: float
        The non-GR inspiral parameter at -1PN
    dchi_minus1: float
        The non-GR inspiral parameter at -0.5PN
    dchi_0: float
        The non-GR inspiral parameter at 0PN
    dchi_1: float
        A non-GR inspiral parameter at 0.5PN
    dchi_2: float
        A non-GR inspiral parameter at 1PN
    dchi_3: float
        A non-GR inspiral parameter at 1.5PN
    dchi_4: float
        A non-GR inspiral parameter at 2PN
    dchi_5l: float
        A non-GR inspiral parameter at 2.5PN, logarithmic correction
    dchi_6: float
        A non-GR inspiral parameter at 3PN
    dchi_6l: float
        A non-GR inspiral parameter at 3PN, logarithmic correction
    dchi_7: float
        A non-GR inspiral parameter at 3.5PN
    db_1: float
        A non-GR intermediate parameter in PhenomX
    db_2: float
        A non-GR intermediate parameter in PhenomX
    db_3: float
        A non-GR merger-ringdown parameter in PhenomX
    db_4: float
        A non-GR merger-ringdown parameter in PhenomX
    dc_1: float
        A non-GR merger-ringdown parameter in PhenomX
    dc_2: float
        A non-GR merger-ringdown parameter in PhenomX
    dc_4: float
        A non-GR merger-ringdown parameter in PhenomX
    dc_l: float
        A non-GR merger-ringdown parameter in PhenomX
    kwargs: dict
        Optional keyword arguments
        Supported arguments:
            waveform_approximant
            reference_frequency
            minimum_frequency
            maximum_frequency
            catch_waveform_errors
            pn_spin_order
            pn_tidal_order
            pn_phase_order
            pn_amplitude_order
            mode_array:
                Activate a specific mode array and evaluate the model using those
                modes only.  e.g. waveform_arguments =
                dict(waveform_approximant='IMRPhenomHM', mode_array=[[2,2],[2,-2])
                returns the 22 and 2-2 modes only of IMRPhenomHM.  You can only
                specify modes that are included in that particular model.  e.g.
                waveform_arguments = dict(waveform_approximant='IMRPhenomHM',
                mode_array=[[2,2],[2,-2],[5,5],[5,-5]]) is not allowed because the
                55 modes are not included in this model.  Be aware that some models
                only take positive modes and return the positive and the negative
                mode together, while others need to call both.  e.g.
                waveform_arguments = dict(waveform_approximant='IMRPhenomHM',
                mode_array=[[2,2],[4,-4]]) returns the 22 and 2-2 of IMRPhenomHM.
                However, waveform_arguments =
                dict(waveform_approximant='IMRPhenomXHM', mode_array=[[2,2],[4,-4]])
                returns the 22 and 4-4 of IMRPhenomXHM.

    Returns
    -------
    dict: A dictionary with the plus and cross polarisation strain modes
    """
    waveform_kwargs = dict(
        waveform_approximant='IMRPhenomXP', reference_frequency=50.0,
        minimum_frequency=20.0, maximum_frequency=frequency_array[-1],
        catch_waveform_errors=False, pn_spin_order=-1, pn_tidal_order=-1,
        pn_phase_order=-1, pn_amplitude_order=0)
    waveform_kwargs.update(kwargs)

    # If no waveform dictionary is passed, create one
    wf_dict = waveform_kwargs.get("lal_waveform_dictionary", CreateDict())

    # Relative shifts for inspiral phase PN coefficients (absolute value for dchi_minus2, dchi_minus1, and dchi_1)
    if dchi_minus2 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChiMinus2(wf_dict, float(dchi_minus2))
    if dchi_minus1 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChiMinus1(wf_dict, float(dchi_minus1))
    if dchi_0 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi0(wf_dict, float(dchi_0))
    if dchi_1 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi1(wf_dict, float(dchi_1))
    if dchi_2 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi2(wf_dict, float(dchi_2))
    if dchi_3 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi3(wf_dict, float(dchi_3))
    if dchi_4 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi4(wf_dict, float(dchi_4))
    if dchi_5l != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi5L(wf_dict, float(dchi_5l))
    if dchi_6 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi6(wf_dict, float(dchi_6))
    if dchi_6l != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi6L(wf_dict, float(dchi_6l))
    if dchi_7 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi7(wf_dict, float(dchi_7))

    # Relative shifts for intermediate phase coefficients  (PhenomXP/XPHM)
    if db_1 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDB1(wf_dict, float(db_1))
    if db_2 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDB2(wf_dict, float(db_2))
    if db_3 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDB3(wf_dict, float(db_3))
    if db_4 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDB4(wf_dict, float(db_4))

    # Relative shifts for merger-ringdown phase coefficients  (PhenomXP/XPHM)
    if dc_1 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDC1(wf_dict, float(dc_1))
    if dc_2 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDC2(wf_dict, float(dc_2))
    if dc_4 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDC4(wf_dict, float(dc_4))
    if dc_l != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDCL(wf_dict, float(dc_l))

    # Update waveform dictionary with TIGER coefficients
    waveform_kwargs["lal_waveform_dictionary"] = wf_dict

    return _base_lal_cbc_fd_waveform(
        frequency_array=frequency_array, mass_1=mass_1, mass_2=mass_2,
        luminosity_distance=luminosity_distance, theta_jn=theta_jn, phase=phase,
        a_1=a_1, a_2=a_2, tilt_1=tilt_1, tilt_2=tilt_2, phi_12=phi_12,
        phi_jl=phi_jl, **waveform_kwargs)


def lal_binary_neutron_star_TIGER_PhenomP(
        frequency_array, mass_1, mass_2, luminosity_distance, a_1, tilt_1,
        phi_12, a_2, tilt_2, phi_jl, theta_jn, phase, lambda_1, lambda_2,
        dchi_0, dchi_1, dchi_2, dchi_3, dchi_4, dchi_5l, dchi_6, dchi_6l, dchi_7,
        **kwargs):
    """ A Binary Neutron Star waveform model using lalsimulation with
    parameterized corrections in the TIGER framework. This interface
    supports the deviation in PN coefficients.
    PhenomD/P does not support the deviation in the negative PN coefficient.
    Recommended waveform models: IMRPhenomD_NRTidal, IMRPhenomD_NRTidalv2,
    IMRPhenomPv2_NRTidal, IMRPhenomPv2_NRTidalv2
    ----------
    Parameters
    ----------
    frequency_array: array_like
        The frequencies at which we want to calculate the strain
    mass_1: float
        The mass of the heavier object in solar masses
    mass_2: float
        The mass of the lighter object in solar masses
    luminosity_distance: float
        The luminosity distance in megaparsec
    a_1: float
        Dimensionless primary spin magnitude
    tilt_1: float
        Primary tilt angle
    phi_12: float
        Azimuthal angle between the two component spins
    a_2: float
        Dimensionless secondary spin magnitude
    tilt_2: float
        Secondary tilt angle
    phi_jl: float
        Azimuthal angle between the total binary angular momentum and the
        orbital angular momentum
    theta_jn: float
        Orbital inclination
    phase: float
        The phase at coalescence
    lambda_1: float
        Dimensionless tidal deformability of mass_1
    lambda_2: float
        Dimensionless tidal deformability of mass_2
    dchi_0: float
        The non-GR inspiral parameter at 0PN
    dchi_1: float
        A non-GR inspiral parameter at 0.5PN
    dchi_2: float
        A non-GR inspiral parameter at 1PN
    dchi_3: float
        A non-GR inspiral parameter at 1.5PN
    dchi_4: float
        A non-GR inspiral parameter at 2PN
    dchi_5l: float
        A non-GR inspiral parameter at 2.5PN, logarithmic correction
    dchi_6: float
        A non-GR inspiral parameter at 3PN
    dchi_6l: float
        A non-GR inspiral parameter at 3PN, logarithmic correction
    dchi_7: float
        A non-GR inspiral parameter at 3.5PN
    kwargs: dict
        Optional keyword arguments
        Supported arguments:
            waveform_approximant
            reference_frequency
            minimum_frequency
            maximum_frequency
            catch_waveform_errors
            pn_spin_order
            pn_tidal_order
            pn_phase_order
            pn_amplitude_order

    Returns
    -------
    dict: A dictionary with the plus and cross polarisation strain modes
    """
    waveform_kwargs = dict(
        waveform_approximant='IMRPhenomPv2_NRTidal', reference_frequency=50.0,
        minimum_frequency=20.0, maximum_frequency=frequency_array[-1],
        catch_waveform_errors=False, pn_spin_order=-1, pn_tidal_order=-1,
        pn_phase_order=-1, pn_amplitude_order=0)

    waveform_kwargs.update(kwargs)

    # If no waveform dictionary is passed, create one
    wf_dict = waveform_kwargs.get("lal_waveform_dictionary", CreateDict())

    # Relative shifts for inspiral phase PN coefficients (absolute value for dchi_1)
    if dchi_0 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi0(wf_dict, float(dchi_0))
    if dchi_1 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi1(wf_dict, float(dchi_1))
    if dchi_2 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi2(wf_dict, float(dchi_2))
    if dchi_3 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi3(wf_dict, float(dchi_3))
    if dchi_4 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi4(wf_dict, float(dchi_4))
    if dchi_5l != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi5L(wf_dict, float(dchi_5l))
    if dchi_6 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi6(wf_dict, float(dchi_6))
    if dchi_6l != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi6L(wf_dict, float(dchi_6l))
    if dchi_7 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi7(wf_dict, float(dchi_7))

    # Update waveform dictionary with TIGER coefficients
    waveform_kwargs["lal_waveform_dictionary"] = wf_dict

    return _base_lal_cbc_fd_waveform(
        frequency_array=frequency_array, mass_1=mass_1, mass_2=mass_2,
        luminosity_distance=luminosity_distance, theta_jn=theta_jn, phase=phase,
        a_1=a_1, a_2=a_2, tilt_1=tilt_1, tilt_2=tilt_2, phi_12=phi_12,
        phi_jl=phi_jl, lambda_1=lambda_1, lambda_2=lambda_2, **waveform_kwargs)


def lal_binary_neutron_star_TIGER_PhenomX(
        frequency_array, mass_1, mass_2, luminosity_distance, a_1, tilt_1,
        phi_12, a_2, tilt_2, phi_jl, theta_jn, phase, lambda_1, lambda_2,
        dchi_minus2, dchi_minus1, dchi_0, dchi_1, dchi_2, dchi_3, dchi_4, dchi_5l, dchi_6, dchi_6l, dchi_7,
        **kwargs):
    """ A Binary Neutron Star waveform model using lalsimulation with
    parameterized corrections in the TIGER framework. This interface
    supports the PhenomX family with deviation in PN coefficients and
    also, in the negative PN terms (-1PN and -0.5PN).
    Recommended waveform models: IMRPhenomXAS_NRTidalv2, IMRPhenomXP_NRTidalv2
    ----------
    Parameters
    ----------
    frequency_array: array_like
        The frequencies at which we want to calculate the strain
    mass_1: float
        The mass of the heavier object in solar masses
    mass_2: float
        The mass of the lighter object in solar masses
    luminosity_distance: float
        The luminosity distance in megaparsec
    a_1: float
        Dimensionless primary spin magnitude
    tilt_1: float
        Primary tilt angle
    phi_12: float
        Azimuthal angle between the two component spins
    a_2: float
        Dimensionless secondary spin magnitude
    tilt_2: float
        Secondary tilt angle
    phi_jl: float
        Azimuthal angle between the total binary angular momentum and the
        orbital angular momentum
    theta_jn: float
        Orbital inclination
    phase: float
        The phase at coalescence
    lambda_1: float
        Dimensionless tidal deformability of mass_1
    lambda_2: float
        Dimensionless tidal deformability of mass_2
    dchi_minus2: float
        The non-GR inspiral parameter at -1PN
    dchi_minus1: float
        The non-GR inspiral parameter at -0.5PN
    dchi_0: float
        The non-GR inspiral parameter at 0PN
    dchi_1: float
        A non-GR inspiral parameter at 0.5PN
    dchi_2: float
        A non-GR inspiral parameter at 1PN
    dchi_3: float
        A non-GR inspiral parameter at 1.5PN
    dchi_4: float
        A non-GR inspiral parameter at 2PN
    dchi_5l: float
        A non-GR inspiral parameter at 2.5PN, logarithmic correction
    dchi_6: float
        A non-GR inspiral parameter at 3PN
    dchi_6l: float
        A non-GR inspiral parameter at 3PN, logarithmic correction
    dchi_7: float
        A non-GR inspiral parameter at 3.5PN
    kwargs: dict
        Optional keyword arguments
        Supported arguments:
            waveform_approximant
            reference_frequency
            minimum_frequency
            maximum_frequency
            catch_waveform_errors
            pn_spin_order
            pn_tidal_order
            pn_phase_order
            pn_amplitude_order

    Returns
    -------
    dict: A dictionary with the plus and cross polarisation strain modes
    """
    waveform_kwargs = dict(
        waveform_approximant='IMRPhenomXP_NRTidalv2', reference_frequency=50.0,
        minimum_frequency=20.0, maximum_frequency=frequency_array[-1],
        catch_waveform_errors=False, pn_spin_order=-1, pn_tidal_order=-1,
        pn_phase_order=-1, pn_amplitude_order=0)

    waveform_kwargs.update(kwargs)

    # If no waveform dictionary is passed, create one
    wf_dict = waveform_kwargs.get("lal_waveform_dictionary", CreateDict())

    # Relative shifts for inspiral phase PN coefficients (absolute value for dchi_minus2, dchi_minus1, and dchi_1)
    if dchi_minus2 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChiMinus2(wf_dict, float(dchi_minus2))
    if dchi_minus1 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChiMinus1(wf_dict, float(dchi_minus1))
    if dchi_0 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi0(wf_dict, float(dchi_0))
    if dchi_1 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi1(wf_dict, float(dchi_1))
    if dchi_2 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi2(wf_dict, float(dchi_2))
    if dchi_3 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi3(wf_dict, float(dchi_3))
    if dchi_4 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi4(wf_dict, float(dchi_4))
    if dchi_5l != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi5L(wf_dict, float(dchi_5l))
    if dchi_6 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi6(wf_dict, float(dchi_6))
    if dchi_6l != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi6L(wf_dict, float(dchi_6l))
    if dchi_7 != 0.:
        lalsimulation.SimInspiralWaveformParamsInsertNonGRDChi7(wf_dict, float(dchi_7))

    # Update waveform dictionary with TIGER coefficients
    waveform_kwargs["lal_waveform_dictionary"] = wf_dict

    return _base_lal_cbc_fd_waveform(
        frequency_array=frequency_array, mass_1=mass_1, mass_2=mass_2,
        luminosity_distance=luminosity_distance, theta_jn=theta_jn, phase=phase,
        a_1=a_1, a_2=a_2, tilt_1=tilt_1, tilt_2=tilt_2, phi_12=phi_12,
        phi_jl=phi_jl, lambda_1=lambda_1, lambda_2=lambda_2, **waveform_kwargs)



import unittest
from copy import copy
import bilby
import bilby_tgr


class TestLalSIQM(unittest.TestCase):
    def setUp(self):
        self.parameters = dict(
            mass_1=20.0,
            mass_2=14.0,
            luminosity_distance=400.0,
            a_1=0.4,
            a_2=0.3,
            tilt_1=0.2,
            tilt_2=1.7,
            phi_jl=0.2,
            phi_12=0.9,
            theta_jn=1.7,
            phase=0.0,
            dQuadMon1=100.0,
            dQuadMon2=100.0,
        )
        self.waveform_kwargs = dict(
            waveform_approximant="IMRPhenomPv2",
            reference_frequency=50.0,
            catch_waveform_errors=True,
            minimum_frequency=20.0,
        )
        self.frequency_array = bilby.core.utils.create_frequency_series(2048, 4)
        self.bad_parameters = copy(self.parameters)
        self.bad_parameters["mass_1"] = -30.0

    def tearDown(self):
        del self.parameters
        del self.waveform_kwargs
        del self.frequency_array

    def test_waveform_error_catching_siqm(self):
        self.bad_parameters.update(self.waveform_kwargs)
        self.assertIsNone(
            bilby_tgr.siqm.source.lal_binary_black_hole(
                self.frequency_array, **self.bad_parameters
            )
        )

    def test_waveform_error_raising_siqm(self):
        raise_error_parameters = copy(self.bad_parameters)
        raise_error_parameters.update(self.waveform_kwargs)
        raise_error_parameters["catch_waveform_errors"] = False
        with self.assertRaises(Exception):
            bilby_tgr.siqm.source.lal_binary_black_hole(
                self.frequency_array, **raise_error_parameters
            )

    def test_lal_binary_black_hole_works_without_siqm_parameters(self):
        self.parameters.pop("dQuadMon1")
        self.parameters.pop("dQuadMon2")
        self.parameters.update(self.waveform_kwargs)
        with self.assertRaises(TypeError):
            bilby_tgr.siqm.source.lal_binary_black_hole(
                self.frequency_array, **self.parameters
            )

    def test_lal_binary_black_hole_runs_with_valid_parameters(self):
        self.parameters.update(self.waveform_kwargs)
        self.assertIsInstance(
            bilby_tgr.siqm.source.lal_binary_black_hole(
                self.frequency_array, **self.parameters
            ),
            dict,
        )


if __name__ == "__main__":
    unittest.main()



import unittest

import pandas as pd

import bilby
import bilby_tgr


class TestBasicConversions(unittest.TestCase):
    def setUp(self):
        self.dchi_kappaS = 150.
        self.dchi_kappaA = -50.
        self.dchi_kappa1 = 100.
        self.dchi_kappa2 = 200.

    def tearDown(self):
        del self.dchi_kappaS
        del self.dchi_kappaA
        del self.dchi_kappa1
        del self.dchi_kappa2

    def test_dkappa1_and_dkappa2_to_dkappaS_and_dkappaA(self):
        dchi_kappaS, dchi_kappaA = bilby_tgr.fti.conversion.dkappa1_and_dkappa2_to_dkappaS_and_dkappaA(
            self.dchi_kappa1, self.dchi_kappa2
        )
        self.assertAlmostEqual(self.dchi_kappaS, dchi_kappaS)
        self.assertAlmostEqual(self.dchi_kappaA, dchi_kappaA)

    def test_dkappaS_and_dkappaA_to_dkappa1_and_dkappa2(self):
        dchi_kappa1, dchi_kappa2 = bilby_tgr.fti.conversion.dkappaS_and_dkappaA_to_dkappa1_and_dkappa2(
            self.dchi_kappaS, self.dchi_kappaA
        )
        self.assertAlmostEqual(self.dchi_kappa1, dchi_kappa1)
        self.assertAlmostEqual(self.dchi_kappa2, dchi_kappa2)


class TestConvertToLALParams(unittest.TestCase):
    def setUp(self):
        self.parameters = dict()

    def tearDown(self):
        del self.parameters

    def bbh_convert(self):
        (
            self.parameters,
            self.added_keys,
        ) = bilby_tgr.fti.conversion.convert_to_lal_binary_black_hole_parameters(self.parameters)

    def test_dkappa1_and_dkappa2_to_dkappaS_and_dkappaA(self):
        self.parameters["dchi_kappa1"] = 100
        self.parameters["dchi_kappa2"] = 200
        dchi_kappaS, dchi_kappaA = bilby_tgr.fti.conversion.dkappa1_and_dkappa2_to_dkappaS_and_dkappaA(
            self.parameters["dchi_kappa1"], self.parameters["dchi_kappa2"]
        )
        self.bbh_convert()
        self.assertAlmostEqual(self.parameters["dchi_kappaS"], dchi_kappaS)
        self.assertAlmostEqual(self.parameters["dchi_kappaA"], dchi_kappaA)


class TestGenerateAllParameters(unittest.TestCase):
    def setUp(self):
        self.parameters = dict(
            mass_1=36.0,
            mass_2=29.0,
            a_1=0.4,
            a_2=0.3,
            tilt_1=0.5,
            tilt_2=1.0,
            phi_12=1.7,
            phi_jl=0.3,
            luminosity_distance=2000.0,
            theta_jn=0.4,
            psi=2.659,
            phase=1.3,
            geocent_time=1126259642.413,
            ra=1.375,
            dec=-1.2108,
            lambda_tilde=1000,
            delta_lambda_tilde=0,
            dchi_Minus2=0.1,
            dchi_0=0.1,
            dchi_1=0.1,
            dchi_2=0.1,
            dchi_3=0.1,
            dchi_3S=0.1,
            dchi_3NS=0.1,
            dchi_4=0.1,
            dchi_4S=0.1,
            dchi_4NS=0.1,
            dchi_5l=0.1,
            dchi_5lS=0.1,
            dchi_5lNS=0.1,
            dchi_6=0.1,
            dchi_6S=0.1,
            dchi_6NS=0.1,
            dchi_6l=0.1,
            dchi_7=0.1,
            dchi_7S=0.1,
            dchi_7NS=0.1,
            dchi_kappaS=0.1,
            dchi_kappaA=0.1,
            f_window_div_f_peak=1.0,
            NumCycles=1,
        )
        self.expected_bbh_keys = [
            "mass_1",
            "mass_2",
            "a_1",
            "a_2",
            "tilt_1",
            "tilt_2",
            "phi_12",
            "phi_jl",
            "luminosity_distance",
            "theta_jn",
            "psi",
            "phase",
            "geocent_time",
            "ra",
            "dec",
            "reference_frequency",
            "waveform_approximant",
            "minimum_frequency",
            "chirp_mass",
            "total_mass",
            "symmetric_mass_ratio",
            "mass_ratio",
            "iota",
            "spin_1x",
            "spin_1y",
            "spin_1z",
            "spin_2x",
            "spin_2y",
            "spin_2z",
            "phi_1",
            "phi_2",
            "chi_eff",
            "chi_1_in_plane",
            "chi_2_in_plane",
            "chi_p",
            "cos_tilt_1",
            "cos_tilt_2",
            "redshift",
            "comoving_distance",
            "mass_1_source",
            "mass_2_source",
            "chirp_mass_source",
            "total_mass_source",
            "dchi_Minus2",
            "dchi_0",
            "dchi_1",
            "dchi_2",
            "dchi_3",
            "dchi_3S",
            "dchi_3NS",
            "dchi_4",
            "dchi_4S",
            "dchi_4NS",
            "dchi_5l",
            "dchi_5lS",
            "dchi_5lNS",
            "dchi_6",
            "dchi_6S",
            "dchi_6NS",
            "dchi_6l",
            "dchi_7",
            "dchi_7S",
            "dchi_7NS",
            "dchi_kappaS",
            "dchi_kappaA",
            "dchi_kappa1",
            "dchi_kappa2",
            "f_window_div_f_peak",
            "NumCycles",
        ]
        self.expected_tidal_keys = [
            "lambda_1",
            "lambda_2",
            "lambda_tilde",
            "delta_lambda_tilde",
        ]
        self.data_frame = pd.DataFrame({
            key: [value] * 100 for key, value in self.parameters.items()
        })

    def test_generate_all_bbh_parameters(self):
        self._generate(
            bilby_tgr.fti.conversion.generate_all_bbh_parameters,
            self.expected_bbh_keys,
        )

    def test_generate_all_bns_parameters(self):
        self._generate(
            bilby_tgr.fti.conversion.generate_all_bns_parameters,
            self.expected_bbh_keys + self.expected_tidal_keys,
        )

    def _generate(self, func, expected):
        for values in [self.parameters, self.data_frame]:
            new_parameters = func(values)
            for key in expected:
                self.assertIn(key, new_parameters)

    def test_generate_bbh_parameters_with_likelihood(self):
        priors = bilby_tgr.fti.prior.BBHPriorDict()
        priors["geocent_time"] = bilby.core.prior.Uniform(0.4, 0.6)
        priors["dchi_kappaS"] = bilby.core.prior.Uniform(-500, 500)
        ifos = bilby.gw.detector.InterferometerList(["H1"])
        ifos.set_strain_data_from_power_spectral_densities(duration=1, sampling_frequency=256)
        wfg = bilby.gw.waveform_generator.WaveformGenerator(
            frequency_domain_source_model=bilby_tgr.fti.source.lal_binary_black_hole
        )
        likelihood = bilby.gw.likelihood.GravitationalWaveTransient(
            interferometers=ifos,
            waveform_generator=wfg,
            priors=priors,
            phase_marginalization=True,
            time_marginalization=True,
            reference_frame="H1L1",
        )
        self.parameters["zenith"] = 0.0
        self.parameters["azimuth"] = 0.0
        self.parameters["time_jitter"] = 0.0
        del self.parameters["ra"], self.parameters["dec"]
        self.parameters = pd.DataFrame(self.parameters, index=range(1))
        converted = bilby_tgr.fti.conversion.generate_all_bbh_parameters(
            sample=self.parameters, likelihood=likelihood, priors=priors
        )
        extra_expected = [
            "geocent_time",
            "phase",
            "H1_optimal_snr",
            "H1_matched_filter_snr",
            "ra",
            "dec",
            "dchi_kappa1",
            "dchi_kappa2",
        ]
        for key in extra_expected:
            self.assertIn(key, converted)


if __name__ == "__main__":
    unittest.main()



import bilby
from scipy.constants import physical_constants
import numpy as np
from bilby.core.utils.constants import speed_of_light as c, parsec as pc, solar_mass, gravitational_constant as G


h = physical_constants["Planck constant in eV/Hz"][0] * 1e12  # h in peV*s


def liv_correction_group_velocity(
        frequency_array, luminosity_distance, alpha, A_eff, **kwargs):
    """
    calculates LIV correction using group velocity following https://arxiv.org/pdf/2203.13252.pdf

    Parameters:


    frequency_array: frequencies on which to evaluate the correction, in Hz
    luminosity_distance: float
        luminosity distance in Mpc
    alpha: float
        exponent of lorentz violating momentum term
    A_eff: float
        A_eff in units of peV^(2-alpha)

    Output

    correction: np.array
        LIV phase correction factor, so that the overall modification is exp(1j*correction)"""

    distance = luminosity_distance * pc * 1e6
    zeta = -(np.pi * distance * A_eff * h**(alpha - 2.) / c)
    return zeta * frequency_array**(alpha - 1.)


def liv_correction_phase_velocity(
        frequency_array, mass_1, mass_2, luminosity_distance, alpha, A_eff, **kwargs):
    """
    calculates LIV correction using phase velocity following https://arxiv.org/pdf/1110.2720.pdf,
    but parametrized in A_eff

    Parameters:


    frequency_array: frequencies on which to evaluate the correction, in Hz
    mass_1, mass_2: masses of the binary, in solar mass
    luminosity_distance: float
        luminosity distance in Mpc
    alpha: float
        exponent of lorentz violating momentum term
    A_eff: float
        A_eff in units of peV^(2-alpha)

    Output

    correction: np.array
        LIV phase correction factor, so that the overall modification is exp(1j*correction)"""

    if (alpha == 1.):
        distance = luminosity_distance * pc * 1e6
        zeta = np.pi * distance * A_eff / h / c
        mchirp = bilby.gw.conversion.component_masses_to_chirp_mass(mass_1, mass_2) * solar_mass
        freq_factor = np.pi * mchirp * G / c**3
        return zeta * np.log(freq_factor * frequency_array)
    else:
        distance = luminosity_distance * pc * 1e6
        zeta = (np.pi * distance * A_eff / (alpha - 1.) * h**(alpha - 2.) / c)
        return zeta * frequency_array**(alpha - 1.)


def lal_binary_black_hole(
        frequency_array, mass_1, mass_2, luminosity_distance, a_1, tilt_1,
        phi_12, a_2, tilt_2, phi_jl, theta_jn, phase, alpha, A_eff, **kwargs):
    """
    Binary black hole model with group velocity LIV correction

    alpha: float
        exponent of lorentz violating momentum term
    A_eff: float
        A_eff in units of peV^(2-alpha)
    """

    gr_waveform = bilby.gw.source.lal_binary_black_hole(
        frequency_array=frequency_array, mass_1=mass_1, mass_2=mass_2, luminosity_distance=luminosity_distance,
        a_1=a_1, tilt_1=tilt_1, phi_12=phi_12, a_2=a_2, tilt_2=tilt_2, phi_jl=phi_jl,
        theta_jn=theta_jn, phase=phase, **kwargs)

    correction = np.zeros_like(frequency_array, dtype=complex)
    if correction[0] == 0:  # avoid raising 0 to negative power
        correction[1:] = np.exp(liv_correction_group_velocity(
            frequency_array[1:], luminosity_distance, alpha, A_eff, **kwargs) * 1j)
    else:
        correction = np.exp(liv_correction_group_velocity(
            frequency_array, luminosity_distance, alpha, A_eff, **kwargs) * 1j)

    if gr_waveform is None:
        return None
    else:
        return dict(plus=gr_waveform['plus'] * correction, cross=gr_waveform['cross'] * correction)


def lal_binary_neutron_star(
        frequency_array, mass_1, mass_2, luminosity_distance, a_1, tilt_1,
        phi_12, a_2, tilt_2, phi_jl, theta_jn, phase, lambda_1, lambda_2, alpha, A_eff, **kwargs):
    """
    Binary neutron star model with group velocity LIV correction

    alpha: float
        exponent of lorentz violating momentum term
    A_eff: float
        A_eff in units of peV^(2-alpha)
    """

    gr_waveform = bilby.gw.source.lal_binary_neutron_star(
        frequency_array=frequency_array, mass_1=mass_1, mass_2=mass_2, luminosity_distance=luminosity_distance,
        a_1=a_1, tilt_1=tilt_1, phi_12=phi_12, a_2=a_2, tilt_2=tilt_2, phi_jl=phi_jl,
        theta_jn=theta_jn, phase=phase, lambda_1=lambda_1, lambda_2=lambda_2, **kwargs)

    correction = np.zeros_like(frequency_array, dtype=complex)
    if correction[0] == 0:  # avoid raising 0 to negative power
        correction[1:] = np.exp(liv_correction_group_velocity(
            frequency_array[1:], luminosity_distance, alpha, A_eff, **kwargs) * 1j)
    else:
        correction = np.exp(liv_correction_group_velocity(
            frequency_array, luminosity_distance, alpha, A_eff, **kwargs) * 1j)

    if gr_waveform is None:
        return None
    else:
        return dict(plus=gr_waveform['plus'] * correction, cross=gr_waveform['cross'] * correction)


def lal_binary_black_hole_phase_velocity(
        frequency_array, mass_1, mass_2, luminosity_distance, a_1, tilt_1,
        phi_12, a_2, tilt_2, phi_jl, theta_jn, phase, alpha, A_eff, **kwargs):
    """
    Binary black hole model with phase velocity LIV correction

    alpha: float
        exponent of lorentz violating momentum term
    A_eff: float
        A_eff in units of peV^(2-alpha)
    """

    gr_waveform = bilby.gw.source.lal_binary_black_hole(
        frequency_array=frequency_array, mass_1=mass_1, mass_2=mass_2, luminosity_distance=luminosity_distance,
        a_1=a_1, tilt_1=tilt_1, phi_12=phi_12, a_2=a_2, tilt_2=tilt_2, phi_jl=phi_jl,
        theta_jn=theta_jn, phase=phase, **kwargs)

    correction = np.zeros_like(frequency_array, dtype=complex)
    if correction[0] == 0:  # avoid raising 0 to negative power
        correction[1:] = np.exp(liv_correction_phase_velocity(
            frequency_array[1:], mass_1, mass_2, luminosity_distance, alpha, A_eff, **kwargs) * 1j)
    else:
        correction = np.exp(liv_correction_phase_velocity(
            frequency_array, mass_1, mass_2, luminosity_distance, alpha, A_eff, **kwargs) * 1j)

    if isinstance(gr_waveform, type(None)):
        return None
    else:
        return dict(plus=gr_waveform['plus'] * correction, cross=gr_waveform['cross'] * correction)


def lal_binary_neutron_star_phase_velocity(
        frequency_array, mass_1, mass_2, luminosity_distance, a_1, tilt_1,
        phi_12, a_2, tilt_2, phi_jl, theta_jn, phase, lambda_1, lambda_2, alpha, A_eff, **kwargs):
    """
    Binary neutron star model with phase velocity LIV correction

    alpha: float
        exponent of lorentz violating momentum term
    A_eff: float
        A_eff in units of peV^(2-alpha)
    """

    gr_waveform = bilby.gw.source.lal_binary_neutron_star(
        frequency_array=frequency_array, mass_1=mass_1, mass_2=mass_2, luminosity_distance=luminosity_distance,
        a_1=a_1, tilt_1=tilt_1, phi_12=phi_12, a_2=a_2, tilt_2=tilt_2, phi_jl=phi_jl,
        theta_jn=theta_jn, phase=phase, lambda_1=lambda_1, lambda_2=lambda_2, **kwargs)

    correction = np.zeros_like(frequency_array, dtype=complex)
    if correction[0] == 0:  # avoid raising 0 to negative power
        correction[1:] = np.exp(liv_correction_phase_velocity(
            frequency_array[1:], mass_1, mass_2, luminosity_distance, alpha, A_eff, **kwargs) * 1j)
    else:
        correction = np.exp(liv_correction_phase_velocity(
            frequency_array, mass_1, mass_2, luminosity_distance, alpha, A_eff, **kwargs) * 1j)

    if isinstance(gr_waveform, type(None)):
        return None
    else:
        return dict(plus=gr_waveform['plus'] * correction, cross=gr_waveform['cross'] * correction)



import unittest

import numpy as np
from copy import copy

import bilby
import bilby_tgr


class TestSsbBBH(unittest.TestCase):
    def setUp(self):
        self.parameters = dict(
            mass_1=30.0,
            mass_2=30.0,
            luminosity_distance=400.0,
            a_1=0.4,
            tilt_1=0.2,
            phi_12=1.0,
            a_2=0.8,
            tilt_2=2.7,
            phi_jl=2.9,
            theta_jn=0.3,
            phase=0.0,
            ssb_coeff=8000,
        )
        self.waveform_kwargs = dict(
            waveform_approximant="IMRPhenomPv2",
            reference_frequency=50.0,
            minimum_frequency=20.0,
            catch_waveform_errors=True,
        )
        self.frequency_array = bilby.core.utils.create_frequency_series(2048, 4)
        self.bad_parameters = copy(self.parameters)
        self.bad_parameters["mass_1"] = -30.0

    def tearDown(self):
        del self.parameters
        del self.waveform_kwargs
        del self.frequency_array
        del self.bad_parameters

    def test_ssb_bbh_works_runs_valid_parameters(self):
        self.parameters.update(self.waveform_kwargs)
        self.assertIsInstance(
            bilby_tgr.ssb.source.lal_binary_black_hole(
                self.frequency_array, **self.parameters
            ),
            dict,
        )

    def test_waveform_error_catching(self):
        self.bad_parameters.update(self.waveform_kwargs)
        self.assertIsNone(
            bilby_tgr.ssb.source.lal_binary_black_hole(
                self.frequency_array, **self.bad_parameters
            )
        )

    def test_waveform_error_raising(self):
        raise_error_parameters = copy(self.bad_parameters)
        raise_error_parameters.update(self.waveform_kwargs)
        raise_error_parameters["catch_waveform_errors"] = False
        with self.assertRaises(Exception):
            bilby_tgr.ssb.source.lal_binary_black_hole(
                self.frequency_array, **raise_error_parameters
            )

    def test_ssb_bbh_works_without_waveform_parameters(self):
        self.assertIsInstance(
            bilby_tgr.ssb.source.lal_binary_black_hole(
                self.frequency_array, **self.parameters
            ),
            dict,
        )

    def test_ssb_bbh_xpprecession_version(self):
        self.parameters.update(self.waveform_kwargs)
        self.parameters["waveform_approximant"] = "IMRPhenomXP"

        # Test that we can modify the XP precession version
        out_v223 = bilby_tgr.ssb.source.lal_binary_black_hole(
            self.frequency_array, PhenomXPrecVersion=223, **self.parameters
        )
        out_v102 = bilby_tgr.ssb.source.lal_binary_black_hole(
            self.frequency_array, PhenomXPrecVersion=102, **self.parameters
        )
        self.assertFalse(np.all(out_v223["plus"] == out_v102["plus"]))


if __name__ == "__main__":
    unittest.main()



from collections import OrderedDict
import unittest
import glob
import os
import pickle

import bilby
import bilby_tgr


class TestBBHPriorDict(unittest.TestCase):
    def setUp(self):
        self.prior_dict = dict()
        self.base_directory = "/".join(
            os.path.dirname(os.path.realpath(__file__)).split("/")[:-2]
        )
        self.filename = os.path.join(
            self.base_directory,
            "bilby_tgr/fti/prior_files/aligned_spins_bbh.prior",
        )
        self.bbh_prior_dict = bilby_tgr.fti.prior.BBHPriorDict(filename=self.filename)
        for key, value in self.bbh_prior_dict.items():
            self.prior_dict[key] = value

    def tearDown(self):
        del self.prior_dict
        del self.filename
        del self.bbh_prior_dict

    def test_create_default_prior(self):
        default = bilby_tgr.fti.prior.BBHPriorDict()
        minima = all(
            [
                self.bbh_prior_dict[key].minimum == default[key].minimum
                for key in default.keys()
            ]
        )
        maxima = all(
            [
                self.bbh_prior_dict[key].maximum == default[key].maximum
                for key in default.keys()
            ]
        )
        names = all(
            [
                self.bbh_prior_dict[key].name == default[key].name
                for key in default.keys()
            ]
        )
        boundaries = all(
            [
                self.bbh_prior_dict[key].boundary == default[key].boundary
                for key in default.keys()
            ]
        )

        self.assertTrue(all([minima, maxima, names, boundaries]))

    def test_create_from_dict(self):
        new_dict = bilby_tgr.fti.prior.BBHPriorDict(dictionary=self.prior_dict)
        for key in self.bbh_prior_dict:
            self.assertEqual(self.bbh_prior_dict[key], new_dict[key])

    def test_redundant_priors_not_in_dict_before(self):
        for prior in [
            "chirp_mass",
            "total_mass",
            "mass_ratio",
            "symmetric_mass_ratio",
            "cos_theta_jn",
            "comoving_distance",
            "redshift",
            "dchi_kappa1",
            "dchi_kappa2",
        ]:
            self.assertTrue(self.bbh_prior_dict.test_redundancy(prior))

    def test_redundant_priors_already_in_dict(self):
        for prior in [
            "mass_1",
            "mass_2",
            "theta_jn",
            "luminosity_distance",
            "dchi_kappaS",
            "dchi_kappaA",
        ]:
            self.assertTrue(self.bbh_prior_dict.test_redundancy(prior))

    def test_correct_not_redundant_priors_masses(self):
        del self.bbh_prior_dict["chirp_mass"]
        for prior in ["chirp_mass", "total_mass", "symmetric_mass_ratio"]:
            self.assertFalse(self.bbh_prior_dict.test_redundancy(prior))

    def test_correct_not_redundant_priors_spin_magnitudes(self):
        del self.bbh_prior_dict["chi_2"]
        self.assertFalse(self.bbh_prior_dict.test_redundancy("chi_2"))

    def test_correct_not_redundant_priors_inclination(self):
        del self.bbh_prior_dict["theta_jn"]
        for prior in ["theta_jn", "cos_theta_jn"]:
            self.assertFalse(self.bbh_prior_dict.test_redundancy(prior))

    def test_correct_not_redundant_priors_distance(self):
        del self.bbh_prior_dict["luminosity_distance"]
        for prior in ["luminosity_distance", "comoving_distance", "redshift"]:
            self.assertFalse(self.bbh_prior_dict.test_redundancy(prior))

    def test_correct_not_redundant_priors_dchi_kappas(self):
        del self.bbh_prior_dict["dchi_kappaA"]
        self.assertFalse(self.bbh_prior_dict.test_redundancy("dchi_kappaA"))

    def test_add_unrelated_prior(self):
        self.assertFalse(self.bbh_prior_dict.test_redundancy("abc"))

    def test_test_has_redundant_priors(self):
        self.assertFalse(self.bbh_prior_dict.test_has_redundant_keys())
        for prior in [
            "mass_1",
            "mass_2",
            "total_mass",
            "symmetric_mass_ratio",
            "cos_theta_jn",
            "comoving_distance",
            "redshift",
            "dchi_kappa1",
            "dchi_kappa2",
        ]:
            self.bbh_prior_dict[prior] = 0
            self.assertTrue(self.bbh_prior_dict.test_has_redundant_keys())
            del self.bbh_prior_dict[prior]

    def test_add_constraint_prior_not_redundant(self):
        self.bbh_prior_dict["dchi_kappaA"] = bilby.prior.Constraint(
            minimum=20, maximum=40, name="dchi_kappaA"
        )
        self.assertFalse(self.bbh_prior_dict.test_has_redundant_keys())

    def test_pickle_prior(self):
        priors = dict(
            chirp_mass=bilby.core.prior.Uniform(10, 20),
            mass_ratio=bilby.core.prior.Uniform(0.125, 1),
        )
        priors = bilby_tgr.fti.prior.BBHPriorDict(priors)
        with open("test.pickle", "wb") as file:
            pickle.dump(priors, file)
        with open("test.pickle", "rb") as file:
            priors_loaded = pickle.load(file)
        self.assertEqual(priors, priors_loaded)
        os.remove("test.pickle")


class TestPackagedPriors(unittest.TestCase):
    """ Test that the prepackaged priors load """

    def test_aligned(self):
        filename = "aligned_spins_bbh.prior"
        prior_dict = bilby_tgr.fti.prior.BBHPriorDict(filename=filename)
        self.assertTrue("chi_1" in prior_dict)
        self.assertTrue("chi_2" in prior_dict)

    def test_binary_black_holes(self):
        filename = "precessing_spins_bbh.prior"
        prior_dict = bilby_tgr.fti.prior.BBHPriorDict(filename=filename)
        self.assertTrue("a_1" in prior_dict)

    def test_all(self):
        prior_files = glob.glob(bilby_tgr.fti.prior.DEFAULT_PRIOR_DIR + "/*prior")
        for ff in prior_files:
            print("Checking prior file {}".format(ff))
            prior_dict = bilby_tgr.fti.prior.BBHPriorDict(filename=ff)
            self.assertTrue("chirp_mass" in prior_dict)
            self.assertTrue("mass_ratio" in prior_dict)
            if "precessing" in ff:
                self.assertTrue("a_1" in prior_dict)
            elif "aligned" in ff:
                self.assertTrue("chi_1" in prior_dict)


class TestBNSPriorDict(unittest.TestCase):
    def setUp(self):
        self.prior_dict = OrderedDict()
        self.base_directory = "/".join(
            os.path.dirname(os.path.realpath(__file__)).split("/")[:-2]
        )
        self.filename = os.path.join(
            self.base_directory,
            "bilby_tgr/fti/prior_files/aligned_spins_bns_tides_on.prior",
        )
        self.bns_prior_dict = bilby_tgr.fti.prior.BNSPriorDict(filename=self.filename)
        for key, value in self.bns_prior_dict.items():
            self.prior_dict[key] = value

    def tearDown(self):
        del self.prior_dict
        del self.filename
        del self.bns_prior_dict
        del self.base_directory

    def test_create_default_prior(self):
        default = bilby_tgr.fti.prior.BNSPriorDict()
        minima = all(
            [
                self.bns_prior_dict[key].minimum == default[key].minimum
                for key in default.keys()
            ]
        )
        maxima = all(
            [
                self.bns_prior_dict[key].maximum == default[key].maximum
                for key in default.keys()
            ]
        )
        names = all(
            [
                self.bns_prior_dict[key].name == default[key].name
                for key in default.keys()
            ]
        )
        boundaries = all(
            [
                self.bns_prior_dict[key].boundary == default[key].boundary
                for key in default.keys()
            ]
        )

        self.assertTrue(all([minima, maxima, names, boundaries]))

    def test_create_from_dict(self):
        new_dict = bilby_tgr.fti.prior.BNSPriorDict(dictionary=self.prior_dict)
        self.assertDictEqual(self.bns_prior_dict, new_dict)

    def test_redundant_priors_not_in_dict_before(self):
        for prior in [
            "chirp_mass",
            "total_mass",
            "mass_ratio",
            "symmetric_mass_ratio",
            "cos_theta_jn",
            "comoving_distance",
            "redshift",
            "lambda_tilde",
            "delta_lambda_tilde",
            "dchi_kappa1",
            "dchi_kappa2",
        ]:
            self.assertTrue(self.bns_prior_dict.test_redundancy(prior))

    def test_redundant_priors_already_in_dict(self):
        for prior in [
            "mass_1",
            "mass_2",
            "chi_1",
            "chi_2",
            "theta_jn",
            "luminosity_distance",
            "lambda_1",
            "lambda_2",
            "dchi_kappaS",
            "dchi_kappaA",
        ]:
            self.assertTrue(self.bns_prior_dict.test_redundancy(prior))

    def test_correct_not_redundant_priors_masses(self):
        del self.bns_prior_dict["chirp_mass"]
        for prior in ["chirp_mass", "total_mass", "symmetric_mass_ratio"]:
            self.assertFalse(self.bns_prior_dict.test_redundancy(prior))

    def test_correct_not_redundant_priors_spin_magnitudes(self):
        del self.bns_prior_dict["chi_2"]
        self.assertFalse(self.bns_prior_dict.test_redundancy("chi_2"))

    def test_correct_not_redundant_priors_inclination(self):
        del self.bns_prior_dict["theta_jn"]
        for prior in ["theta_jn", "cos_theta_jn"]:
            self.assertFalse(self.bns_prior_dict.test_redundancy(prior))

    def test_correct_not_redundant_priors_distance(self):
        del self.bns_prior_dict["luminosity_distance"]
        for prior in ["luminosity_distance", "comoving_distance", "redshift"]:
            self.assertFalse(self.bns_prior_dict.test_redundancy(prior))

    def test_correct_not_redundant_priors_tidal(self):
        del self.bns_prior_dict["lambda_1"]
        for prior in ["lambda_1", "lambda_tilde", "delta_lambda_tilde"]:
            self.assertFalse(self.bns_prior_dict.test_redundancy(prior))

    def test_correct_not_redundant_priors_dchi_kappas(self):
        del self.bns_prior_dict["dchi_kappaA"]
        self.assertFalse(self.bns_prior_dict.test_redundancy("dchi_kappaA"))

    def test_add_unrelated_prior(self):
        self.assertFalse(self.bns_prior_dict.test_redundancy("abc"))

    def test_test_has_redundant_priors(self):
        self.assertFalse(self.bns_prior_dict.test_has_redundant_keys())
        for prior in [
            "mass_1",
            "mass_2",
            "total_mass",
            "symmetric_mass_ratio",
            "cos_theta_jn",
            "comoving_distance",
            "redshift",
            "dchi_kappa1",
            "dchi_kappa2",
        ]:
            self.bns_prior_dict[prior] = 0
            self.assertTrue(self.bns_prior_dict.test_has_redundant_keys())
            del self.bns_prior_dict[prior]

    def test_add_constraint_prior_not_redundant(self):
        self.bns_prior_dict["dchi_kappaA"] = bilby.prior.Constraint(
            minimum=20, maximum=40, name="dchi_kappaA"
        )
        self.assertFalse(self.bns_prior_dict.test_has_redundant_keys())


if __name__ == "__main__":
    unittest.main()



import unittest

import numpy as np
import pandas as pd


import bilby
import bilby_tgr
from bilby_tgr.mdr import conversion


class TestBasicConversionsConsistency(unittest.TestCase):
    # test in back and forth conversion is consistent
    def setUp(self):
        self.alpha = np.array([0., 0.5, 1.5, 2.5, 3.0, 3.5, 4.0])
        self.A_alpha = 1e-20
        self.distance = 1000

    def tearDown(self):
        del self.alpha
        del self.A_alpha
        del self.distance

    def test_A_lambda_conversion(self):
        lam = conversion.A_to_lambda(self.A_alpha, self.alpha)
        A = conversion.lambda_to_A(lam, self.alpha)
        self.assertTrue(
            all(abs(self.A_alpha - A) < 1e-5 * self.A_alpha)
        )

    def test_lambda_lambda_eff_conversion(self):
        lam = conversion.A_to_lambda(self.A_alpha, self.alpha)
        lam_eff = conversion.lambda_to_lambda_eff(lam, self.distance, self.alpha)
        lam_2 = conversion.lambda_eff_to_lambda(lam_eff, self.distance, self.alpha)
        self.assertTrue(
            all(abs(lam - lam_2) < 1e-5 * lam)
        )

    def test_A_lambda_eff_conversion(self):
        lam_eff = conversion.A_to_lambda_eff(self.A_alpha, self.distance, self.alpha)
        A = conversion.lambda_eff_to_A(lam_eff, self.distance, self.alpha)
        self.assertTrue(
            all(abs(self.A_alpha - A) < 1e-5 * self.A_alpha)
        )

    def test_A_A_eff_conversion(self):
        A_eff = conversion.A_to_A_eff(self.A_alpha, self.distance, self.alpha)
        A = conversion.A_eff_to_A(A_eff, self.distance, self.alpha)
        self.assertTrue(
            all(abs(self.A_alpha - A) < 1e-5 * self.A_alpha)
        )


class TestBasicConversions(unittest.TestCase):
    def setUp(self):
        bilby.gw.cosmology.set_cosmology("Planck15")
        self.alpha = 0.0
        self.A_alpha = 1e-20
        self.distance = 2000.
        self.z = 0.3633349489594506
        self.liv_distance = 1481.6623139542735
        self.comoving_distance = 1466.990920476916
        self.lam = 1.239841984055037e+16
        self.lambda_eff = 1.681930332907409e+16
        self.log_lambda_eff = 16.22580800295982
        self.A_eff = 5.43396292703101e-21
        self.sign_A = +1.

    def tearDown(self):
        del self.alpha
        del self.A_alpha
        del self.distance
        del self.z
        del self.liv_distance
        del self.comoving_distance
        del self.lam
        del self.lambda_eff
        del self.log_lambda_eff
        del self.A_eff
        del self.sign_A

    def test_luminosity_distance_to_redshift_scalar(self):
        z = conversion.luminosity_distance_to_redshift_scalar(self.distance)
        self.assertTrue(
            abs(self.z - z) < 1e-5 * self.z
        )

    def test_redshift_to_LIV_distance_scalar(self):
        liv_distance = conversion.redshift_to_LIV_distance_scalar(self.z, self.alpha)
        self.assertTrue(
            abs(self.liv_distance - liv_distance) < 1e-5 * self.liv_distance
        )

    def test_redshift_to_LIV_distance(self):
        liv_distance = conversion.redshift_to_LIV_distance(self.z, self.alpha)
        self.assertTrue(
            abs(self.liv_distance - liv_distance) < 1e-5 * self.liv_distance
        )

    def test_luminosity_distance_to_LIV_distance_scalar(self):
        liv_distance = conversion.luminosity_distance_to_LIV_distance_scalar(self.distance, self.alpha)
        self.assertTrue(
            abs(self.liv_distance - liv_distance) < 1e-5 * self.liv_distance
        )

    def test_luminosity_distance_to_LIV_distance(self):
        liv_distance = conversion.luminosity_distance_to_LIV_distance(self.distance, self.alpha)
        self.assertTrue(
            abs(self.liv_distance - liv_distance) < 1e-5 * self.liv_distance
        )

    def test_comoving_distance_to_LIV_distance(self):
        liv_distance = conversion.comoving_distance_to_LIV_distance(self.comoving_distance, self.alpha)
        self.assertTrue(
            abs(self.liv_distance - liv_distance) < 1e-5 * self.liv_distance
        )

    def test_A_to_lambda(self):
        lam = conversion.A_to_lambda(self.A_alpha, self.alpha)
        self.assertTrue(
            abs(self.lam - lam) < 1e-5 * self.lam
        )

    def test_lambda_to_A(self):
        A_alpha = conversion.lambda_to_A(self.lam, self.alpha)
        self.assertTrue(
            abs(self.A_alpha - A_alpha) < 1e-5 * self.A_alpha
        )

    def test_lambda_to_lambda_eff(self):
        lambda_eff = conversion.lambda_to_lambda_eff(self.lam, self.distance, self.alpha)
        self.assertTrue(
            abs(self.lambda_eff - lambda_eff) < 1e-5 * self.lambda_eff
        )

    def test_lambda_eff_to_lambda(self):
        lam = conversion.lambda_eff_to_lambda(self.lambda_eff, self.distance, self.alpha)
        self.assertTrue(
            abs(self.lam - lam) < 1e-5 * self.lam
        )

    def test_lambda_eff_to_lambda_with_redshift(self):
        lam = conversion.lambda_eff_to_lambda(self.lambda_eff, self.distance, self.alpha, self.z)
        self.assertTrue(
            abs(self.lam - lam) < 1e-5 * self.lam
        )

    def test_A_to_lambda_eff(self):
        lambda_eff = conversion.A_to_lambda_eff(self.A_alpha, self.distance, self.alpha)
        self.assertTrue(
            abs(self.lambda_eff - lambda_eff) < 1e-5 * self.lambda_eff
        )

    def test_lambda_eff_to_A(self):
        A_alpha = conversion.lambda_eff_to_A(self.lambda_eff, self.distance, self.alpha)
        self.assertTrue(
            abs(self.A_alpha - A_alpha) < 1e-5 * self.A_alpha
        )

    def test_lambda_eff_to_A_with_redshift(self):
        A_alpha = conversion.lambda_eff_to_A(self.lambda_eff, self.distance, self.alpha, self.z)
        self.assertTrue(
            abs(self.A_alpha - A_alpha) < 1e-5 * self.A_alpha
        )

    def test_log_lambda_eff_to_A(self):
        A_alpha = conversion.log_lambda_eff_to_A(self.log_lambda_eff, self.distance, self.alpha, self.sign_A)
        self.assertTrue(
            abs(self.A_alpha - A_alpha) < 1e-5 * self.A_alpha
        )

    def test_log_lambda_eff_to_A_with_redshift(self):
        A_alpha = conversion.log_lambda_eff_to_A(self.log_lambda_eff, self.distance, self.alpha, self.sign_A, self.z)
        self.assertTrue(
            abs(self.A_alpha - A_alpha) < 1e-5 * self.A_alpha
        )

    def test_A_to_A_eff(self):
        A_eff = conversion.A_to_A_eff(self.A_alpha, self.distance, self.alpha)
        self.assertTrue(
            abs(self.A_eff - A_eff) < 1e-5 * self.A_eff
        )

    def test_A_eff_to_A(self):
        A_alpha = conversion.A_eff_to_A(self.A_eff, self.distance, self.alpha)
        self.assertTrue(
            abs(self.A_alpha - A_alpha) < 1e-5 * self.A_alpha
        )

    def test_A_eff_to_A_with_redshift(self):
        A_alpha = conversion.A_eff_to_A(self.A_eff, self.distance, self.alpha, self.z)
        self.assertTrue(
            abs(self.A_alpha - A_alpha) < 1e-5 * self.A_alpha
        )


class TestGenerateAllParameters(unittest.TestCase):
    def setUp(self):
        self.parameters = dict(
            mass_1=36.0,
            mass_2=29.0,
            a_1=0.4,
            a_2=0.3,
            tilt_1=0.5,
            tilt_2=1.0,
            phi_12=1.7,
            phi_jl=0.3,
            luminosity_distance=2000.0,
            theta_jn=0.4,
            psi=2.659,
            phase=1.3,
            geocent_time=1126259642.413,
            ra=1.375,
            dec=-1.2108,
            lambda_tilde=1000,
            delta_lambda_tilde=0,
            alpha=0.0,
            A_eff=1e-20,
        )
        self.expected_bbh_keys = [
            "mass_1",
            "mass_2",
            "a_1",
            "a_2",
            "tilt_1",
            "tilt_2",
            "phi_12",
            "phi_jl",
            "luminosity_distance",
            "theta_jn",
            "psi",
            "phase",
            "geocent_time",
            "ra",
            "dec",
            "reference_frequency",
            "waveform_approximant",
            "minimum_frequency",
            "chirp_mass",
            "total_mass",
            "symmetric_mass_ratio",
            "mass_ratio",
            "iota",
            "spin_1x",
            "spin_1y",
            "spin_1z",
            "spin_2x",
            "spin_2y",
            "spin_2z",
            "phi_1",
            "phi_2",
            "chi_eff",
            "chi_1_in_plane",
            "chi_2_in_plane",
            "chi_p",
            "cos_tilt_1",
            "cos_tilt_2",
            "redshift",
            "comoving_distance",
            "mass_1_source",
            "mass_2_source",
            "chirp_mass_source",
            "total_mass_source",
            "alpha",
            "A_eff",
            "A_alpha",
        ]
        self.expected_tidal_keys = [
            "lambda_1",
            "lambda_2",
            "lambda_tilde",
            "delta_lambda_tilde",
        ]
        self.data_frame = pd.DataFrame({
            key: [value] * 100 for key, value in self.parameters.items()
        })

    def test_generate_all_bbh_parameters(self):
        self._generate(
            conversion.generate_all_bbh_parameters,
            self.expected_bbh_keys,
        )

    def test_generate_all_bns_parameters(self):
        self._generate(
            conversion.generate_all_bns_parameters,
            self.expected_bbh_keys + self.expected_tidal_keys,
        )

    def _generate(self, func, expected):
        for values in [self.parameters, self.data_frame]:
            new_parameters = func(values)
            for key in expected:
                self.assertIn(key, new_parameters)

    def test_generate_bbh_parameters_with_likelihood(self):
        priors = bilby.gw.prior.BBHPriorDict()
        priors["geocent_time"] = bilby.core.prior.Uniform(0.4, 0.6)
        priors["alpha"] = 0.0
        priors["A_eff"] = bilby.core.prior.Uniform(-1e-18, 1e-18)
        ifos = bilby.gw.detector.InterferometerList(["H1"])
        ifos.set_strain_data_from_power_spectral_densities(duration=1, sampling_frequency=256)
        wfg = bilby.gw.waveform_generator.WaveformGenerator(
            frequency_domain_source_model=bilby_tgr.mdr.source.lal_binary_black_hole
        )
        likelihood = bilby.gw.likelihood.GravitationalWaveTransient(
            interferometers=ifos,
            waveform_generator=wfg,
            priors=priors,
            phase_marginalization=True,
            time_marginalization=True,
            reference_frame="H1L1",
        )
        self.parameters["zenith"] = 0.0
        self.parameters["azimuth"] = 0.0
        self.parameters["time_jitter"] = 0.0
        del self.parameters["ra"], self.parameters["dec"]
        self.parameters = pd.DataFrame(self.parameters, index=range(1))
        converted = conversion.generate_all_bbh_parameters(
            sample=self.parameters, likelihood=likelihood, priors=priors
        )
        extra_expected = [
            "geocent_time",
            "phase",
            "H1_optimal_snr",
            "H1_matched_filter_snr",
            "ra",
            "dec",
        ]
        for key in extra_expected:
            self.assertIn(key, converted)


if __name__ == "__main__":
    unittest.main()



import unittest

import numpy as np
import pandas as pd


import bilby
import bilby_tgr
from bilby_tgr.ssb import conversion


class TestBasicConversionsConsistency(unittest.TestCase):
    # test in back and forth conversion is consistent
    def setUp(self):
        self.ssb_coeff = np.array([0., 0.5, 1.5, 2.5, 3.0, 3.5, 4.0]) * 1000
        self.distance = 1000

    def tearDown(self):
        del self.ssb_coeff
        del self.distance

    def test_kv5_ssb_coeff_conversion(self):
        redshift = conversion.luminosity_distance_to_redshift(self.distance)
        kv5 = conversion.kv5_from_ssb_coeff(self.ssb_coeff, redshift)
        ssb_coeff = conversion.ssb_coeff_from_kv5_scalar(kv5, redshift)
        self.assertTrue(
            all(abs(self.ssb_coeff - ssb_coeff) <= 1e-5 * self.ssb_coeff)
        )


class TestBasicConversions(unittest.TestCase):
    def setUp(self):
        bilby.gw.cosmology.set_cosmology("Planck15")
        self.ssb_coeff = 5510.6804959978945
        self.kv5 = 3.103992107409165e-14
        self.distance = 2000.
        self.z = 0.3633349489594506

    def tearDown(self):
        del self.ssb_coeff
        del self.kv5
        del self.distance
        del self.z

    def test_luminosity_distance_to_redshift_scalar(self):
        z = conversion.luminosity_distance_to_redshift_scalar(self.distance)
        self.assertTrue(
            abs(self.z - z) <= 1e-5 * self.z
        )

    def test_kv5_from_ssb_coeff_scalar(self):
        kv5 = conversion.kv5_from_ssb_coeff_scalar(self.ssb_coeff, self.z)
        self.assertTrue(
            abs(self.kv5 - kv5) <= 1e-5 * self.kv5
        )

    def test_ssb_coeff_from_kv5_scalar(self):
        ssb_coeff = conversion.ssb_coeff_from_kv5_scalar(self.kv5, self.z)
        self.assertTrue(
            abs(self.ssb_coeff - ssb_coeff) <= 1e-5 * self.ssb_coeff
        )

    def test_luminosity_distance_to_redshift(self):
        z = conversion.luminosity_distance_to_redshift(self.distance)
        self.assertTrue(
            abs(self.z - z) <= 1e-5 * self.z
        )

    def test_kv5_from_ssb_coeff(self):
        kv5 = conversion.kv5_from_ssb_coeff(self.ssb_coeff, self.z)
        self.assertTrue(
            abs(self.kv5 - kv5) <= 1e-5 * self.kv5
        )

    def test_ssb_coeff_from_kv5(self):
        ssb_coeff = conversion.ssb_coeff_from_kv5(self.kv5, self.z)
        self.assertTrue(
            abs(self.ssb_coeff - ssb_coeff) <= 1e-5 * self.ssb_coeff
        )


class TestGenerateAllParameters(unittest.TestCase):
    def setUp(self):
        self.parameters = dict(
            mass_1=36.0,
            mass_2=29.0,
            a_1=0.4,
            a_2=0.3,
            tilt_1=0.5,
            tilt_2=1.0,
            phi_12=1.7,
            phi_jl=0.3,
            luminosity_distance=2000.0,
            theta_jn=0.4,
            psi=2.659,
            phase=1.3,
            geocent_time=1126259642.413,
            ra=1.375,
            dec=-1.2108,
            ssb_coeff=8000,
        )
        self.expected_bbh_keys = [
            "mass_1",
            "mass_2",
            "a_1",
            "a_2",
            "tilt_1",
            "tilt_2",
            "phi_12",
            "phi_jl",
            "luminosity_distance",
            "theta_jn",
            "psi",
            "phase",
            "geocent_time",
            "ra",
            "dec",
            "reference_frequency",
            "waveform_approximant",
            "minimum_frequency",
            "chirp_mass",
            "total_mass",
            "symmetric_mass_ratio",
            "mass_ratio",
            "iota",
            "spin_1x",
            "spin_1y",
            "spin_1z",
            "spin_2x",
            "spin_2y",
            "spin_2z",
            "phi_1",
            "phi_2",
            "chi_eff",
            "chi_1_in_plane",
            "chi_2_in_plane",
            "chi_p",
            "cos_tilt_1",
            "cos_tilt_2",
            "redshift",
            "comoving_distance",
            "mass_1_source",
            "mass_2_source",
            "chirp_mass_source",
            "total_mass_source",
            "ssb_coeff",
            "kv5",
        ]

        self.data_frame = pd.DataFrame({
            key: [value] * 100 for key, value in self.parameters.items()
        })

    def test_generate_all_bbh_parameters(self):
        self._generate(
            conversion.generate_all_bbh_parameters,
            self.expected_bbh_keys,
        )

    def _generate(self, func, expected):
        for values in [self.parameters, self.data_frame]:
            new_parameters = func(values)
            for key in expected:
                self.assertIn(key, new_parameters)

    def test_generate_bbh_parameters_with_likelihood(self):
        priors = bilby.gw.prior.BBHPriorDict()
        priors["geocent_time"] = bilby.core.prior.Uniform(0.4, 0.6)
        priors["ssb_dim"] = 5
        priors["ssb_coeff"] = bilby.core.prior.Uniform(1e-17, 10000)
        ifos = bilby.gw.detector.InterferometerList(["H1"])
        ifos.set_strain_data_from_power_spectral_densities(duration=1, sampling_frequency=256)
        wfg = bilby.gw.waveform_generator.WaveformGenerator(
            frequency_domain_source_model=bilby_tgr.ssb.source.lal_binary_black_hole
        )
        likelihood = bilby.gw.likelihood.GravitationalWaveTransient(
            interferometers=ifos,
            waveform_generator=wfg,
            priors=priors,
            phase_marginalization=True,
            time_marginalization=True,
            reference_frame="H1L1",
        )
        self.parameters["zenith"] = 0.0
        self.parameters["azimuth"] = 0.0
        self.parameters["time_jitter"] = 0.0
        del self.parameters["ra"], self.parameters["dec"]
        self.parameters = pd.DataFrame(self.parameters, index=range(1))
        converted = conversion.generate_all_bbh_parameters(
            sample=self.parameters, likelihood=likelihood, priors=priors
        )
        extra_expected = [
            "geocent_time",
            "phase",
            "H1_optimal_snr",
            "H1_matched_filter_snr",
            "ra",
            "dec",
        ]
        for key in extra_expected:
            self.assertIn(key, converted)


if __name__ == "__main__":
    unittest.main()



import unittest

from copy import copy

import bilby
import bilby_tgr


class TestLalBBH_PhenomP(unittest.TestCase):
    def setUp(self):
        self.parameters = dict(
            mass_1=30.0,
            mass_2=30.0,
            luminosity_distance=400.0,
            a_1=0.4,
            tilt_1=0.2,
            phi_12=1.0,
            a_2=0.8,
            tilt_2=2.7,
            phi_jl=2.9,
            theta_jn=0.3,
            phase=0.0,
            dchi_0=0.1,
            dchi_1=0.1,
            dchi_2=0.1,
            dchi_3=0.1,
            dchi_4=0.1,
            dchi_5l=0.1,
            dchi_6=0.1,
            dchi_6l=0.1,
            dchi_7=0.1,
            dbeta_2=0.1,
            dbeta_3=0.1,
            dalpha_2=0.1,
            dalpha_3=0.1,
            dalpha_4=0.1,
            dalpha_5=0.1,
        )
        self.waveform_kwargs = dict(
            waveform_approximant="IMRPhenomPv3HM",
            reference_frequency=50.0,
            minimum_frequency=20.0,
            catch_waveform_errors=True,
        )
        self.frequency_array = bilby.core.utils.create_frequency_series(2048, 4)
        self.bad_parameters = copy(self.parameters)
        self.bad_parameters["mass_1"] = -30.0

    def tearDown(self):
        del self.parameters
        del self.waveform_kwargs
        del self.frequency_array
        del self.bad_parameters

    def test_lal_bbh_works_runs_valid_parameters(self):
        self.parameters.update(self.waveform_kwargs)
        self.assertIsInstance(
            bilby_tgr.tiger.source.lal_binary_black_hole_TIGER_PhenomP(
                self.frequency_array, **self.parameters
            ),
            dict,
        )

    def test_waveform_error_catching(self):
        self.bad_parameters.update(self.waveform_kwargs)
        self.assertIsNone(
            bilby_tgr.tiger.source.lal_binary_black_hole_TIGER_PhenomP(
                self.frequency_array, **self.bad_parameters
            )
        )

    def test_waveform_error_raising(self):
        raise_error_parameters = copy(self.bad_parameters)
        raise_error_parameters.update(self.waveform_kwargs)
        raise_error_parameters["catch_waveform_errors"] = False
        with self.assertRaises(Exception):
            bilby_tgr.tiger.source.lal_binary_black_hole_TIGER_PhenomP(
                self.frequency_array, **raise_error_parameters
            )

    def test_lal_bbh_works_without_waveform_parameters(self):
        self.assertIsInstance(
            bilby_tgr.tiger.source.lal_binary_black_hole_TIGER_PhenomP(
                self.frequency_array, **self.parameters
            ),
            dict,
        )


class TestLalBBH_PhenomX(unittest.TestCase):
    def setUp(self):
        self.parameters = dict(
            mass_1=30.0,
            mass_2=30.0,
            luminosity_distance=400.0,
            a_1=0.4,
            tilt_1=0.2,
            phi_12=1.0,
            a_2=0.8,
            tilt_2=2.7,
            phi_jl=2.9,
            theta_jn=0.3,
            phase=0.0,
            dchi_minus2=0.1,
            dchi_minus1=0.1,
            dchi_0=0.1,
            dchi_1=0.1,
            dchi_2=0.1,
            dchi_3=0.1,
            dchi_4=0.1,
            dchi_5l=0.1,
            dchi_6=0.1,
            dchi_6l=0.1,
            dchi_7=0.1,
            db_1=0.1,
            db_2=0.1,
            db_3=0.1,
            db_4=0.1,
            dc_1=0.1,
            dc_2=0.1,
            dc_4=0.1,
            dc_l=0.1,
        )
        self.waveform_kwargs = dict(
            waveform_approximant="IMRPhenomXPHM",
            reference_frequency=50.0,
            minimum_frequency=20.0,
            catch_waveform_errors=True,
        )
        self.frequency_array = bilby.core.utils.create_frequency_series(2048, 4)
        self.bad_parameters = copy(self.parameters)
        self.bad_parameters["mass_1"] = -30.0

    def tearDown(self):
        del self.parameters
        del self.waveform_kwargs
        del self.frequency_array
        del self.bad_parameters

    def test_lal_bbh_works_runs_valid_parameters(self):
        self.parameters.update(self.waveform_kwargs)
        self.assertIsInstance(
            bilby_tgr.tiger.source.lal_binary_black_hole_TIGER_PhenomX(
                self.frequency_array, **self.parameters
            ),
            dict,
        )

    def test_waveform_error_catching(self):
        self.bad_parameters.update(self.waveform_kwargs)
        self.assertIsNone(
            bilby_tgr.tiger.source.lal_binary_black_hole_TIGER_PhenomX(
                self.frequency_array, **self.bad_parameters
            )
        )

    def test_waveform_error_raising(self):
        raise_error_parameters = copy(self.bad_parameters)
        raise_error_parameters.update(self.waveform_kwargs)
        raise_error_parameters["catch_waveform_errors"] = False
        with self.assertRaises(Exception):
            bilby_tgr.tiger.source.lal_binary_black_hole_TIGER_PhenomX(
                self.frequency_array, **raise_error_parameters
            )

    def test_lal_bbh_works_without_waveform_parameters(self):
        self.assertIsInstance(
            bilby_tgr.tiger.source.lal_binary_black_hole_TIGER_PhenomX(
                self.frequency_array, **self.parameters
            ),
            dict,
        )


class TestLalBNS_PhenomP(unittest.TestCase):
    def setUp(self):
        self.parameters = dict(
            mass_1=1.4,
            mass_2=1.4,
            luminosity_distance=400.0,
            a_1=0.4,
            a_2=0.3,
            tilt_1=0.2,
            tilt_2=1.7,
            phi_jl=0.2,
            phi_12=0.9,
            theta_jn=1.7,
            phase=0.0,
            lambda_1=100.0,
            lambda_2=100.0,
            dchi_0=0.1,
            dchi_1=0.1,
            dchi_2=0.1,
            dchi_3=0.1,
            dchi_4=0.1,
            dchi_5l=0.1,
            dchi_6=0.1,
            dchi_6l=0.1,
            dchi_7=0.1,
        )
        self.waveform_kwargs = dict(
            waveform_approximant="IMRPhenomPv2_NRTidalv2",
            reference_frequency=50.0,
            minimum_frequency=20.0,
        )
        self.frequency_array = bilby.core.utils.create_frequency_series(2048, 4)

    def tearDown(self):
        del self.parameters
        del self.waveform_kwargs
        del self.frequency_array

    def test_lal_bns_runs_with_valid_parameters(self):
        self.parameters.update(self.waveform_kwargs)
        self.assertIsInstance(
            bilby_tgr.tiger.source.lal_binary_neutron_star_TIGER_PhenomP(
                self.frequency_array, **self.parameters
            ),
            dict,
        )

    def test_lal_bns_works_without_waveform_parameters(self):
        self.assertIsInstance(
            bilby_tgr.tiger.source.lal_binary_neutron_star_TIGER_PhenomP(
                self.frequency_array, **self.parameters
            ),
            dict,
        )

    def test_fails_without_tidal_parameters(self):
        self.parameters.pop("lambda_1")
        self.parameters.pop("lambda_2")
        self.parameters.update(self.waveform_kwargs)
        with self.assertRaises(TypeError):
            bilby_tgr.tiger.source.lal_binary_neutron_star_TIGER_PhenomP(
                self.frequency_array, **self.parameters
            )


class TestLalBNS_PhenomX(unittest.TestCase):
    def setUp(self):
        self.parameters = dict(
            mass_1=1.4,
            mass_2=1.4,
            luminosity_distance=400.0,
            a_1=0.4,
            a_2=0.3,
            tilt_1=0.2,
            tilt_2=1.7,
            phi_jl=0.2,
            phi_12=0.9,
            theta_jn=1.7,
            phase=0.0,
            lambda_1=100.0,
            lambda_2=100.0,
            dchi_minus2=0.1,
            dchi_minus1=0.1,
            dchi_0=0.1,
            dchi_1=0.1,
            dchi_2=0.1,
            dchi_3=0.1,
            dchi_4=0.1,
            dchi_5l=0.1,
            dchi_6=0.1,
            dchi_6l=0.1,
            dchi_7=0.1,
        )
        self.waveform_kwargs = dict(
            waveform_approximant="IMRPhenomXP_NRTidalv2",
            reference_frequency=50.0,
            minimum_frequency=20.0,
        )
        self.frequency_array = bilby.core.utils.create_frequency_series(2048, 4)

    def tearDown(self):
        del self.parameters
        del self.waveform_kwargs
        del self.frequency_array

    def test_lal_bns_runs_with_valid_parameters(self):
        self.parameters.update(self.waveform_kwargs)
        self.assertIsInstance(
            bilby_tgr.tiger.source.lal_binary_neutron_star_TIGER_PhenomX(
                self.frequency_array, **self.parameters
            ),
            dict,
        )

    def test_lal_bns_works_without_waveform_parameters(self):
        self.assertIsInstance(
            bilby_tgr.tiger.source.lal_binary_neutron_star_TIGER_PhenomX(
                self.frequency_array, **self.parameters
            ),
            dict,
        )

    def test_fails_without_tidal_parameters(self):
        self.parameters.pop("lambda_1")
        self.parameters.pop("lambda_2")
        self.parameters.update(self.waveform_kwargs)
        with self.assertRaises(TypeError):
            bilby_tgr.tiger.source.lal_binary_neutron_star_TIGER_PhenomX(
                self.frequency_array, **self.parameters
            )


if __name__ == "__main__":
    unittest.main()
